package net.sf.csutils.core.model;


/**
 * Interface of a registry objects attribute.
 */
public interface ROAttribute {
	public enum Type {
		/**
		 * String type (default)
		 */
		string,
		/**
		 * Boolean type
		 */
		bool,
		/**
		 * Integer type
		 */
		integer,
		/**
		 * Long type
		 */
		longInteger,
		/**
		 * Duration type
		 */
		duration,
		/**
		 * DateTime type
		 */
		dateTime,
		/**
		 * Date type
		 */
		date,
		/**
		 * Number type
		 */
		number,
		/**
		 * Time type
		 */
		time,
		/**
		 * IPV4 address
		 */
		ipv4Address,
		/**
		 * IPV6 address
		 */
		ipv6Address,
		/**
		 * Time type
		 */
		emailAddress,
		/**
		 * Relation type
		 */
		relation,
		/**
		 * URI/URL
		 */
		uri,
		/**
		 * Classification
		 */
		classification,
		/**
		 * File
		 */
		file
	}

	/**
	 * Returns the attribute name.
	 * @return The attribute name (unqualified)
	 */
	String getName();

	/**
	 * Returns the attributes minimum number of occurrencies.
	 * @return Minimum number of occurrencies; 0 for an optional attribute.
	 */
	int getMinOccurs();

	/**
	 * Returns the attributes maximum number of occurrencies.
	 * @return Maximum number or -1 (unbounded)
	 */
	int getMaxOccurs();

	/**
	 * Returns the attribute type.
	 * @return The attributes type.
	 */
	Type getType();
}
