package net.sf.csutils.core.model;

import java.util.Map;

import javax.xml.registry.JAXRException;


/**
 * Interface of a registry object meta model.
 */
public interface ROMetaModel {
	/**
	 * Returns the map of object types. The map keys are the
	 * qualified names, and the map values are the type meta
	 * data objects.
	 * @return The map of object types.
	 * @throws JAXRException The operation failed.
	 */
	Map<QName,ROType> getROTypes() throws JAXRException;

	/**
	 * Returns the object type with the given name.
	 * @param pQName The object types qualified name.
	 * @return Object type, if it exists, or null.
	 * @throws JAXRException The operation failed.
	 */
	ROType getROType(QName pQName) throws JAXRException;
	
	/**
	 * Sets a meta model attribute.
     * @param pKey The attributes key.
     * @param pValue The attributes value.
     * @throws JAXRException The operation failed.
	 */
	void setAttribute(String pKey, Object pValue) throws JAXRException;
	
	/**
	 * Returns a meta model attribute.
	 * @param pKey The attributes key.
	 * @return The attributes value, if any, or null.
     * @throws JAXRException The operation failed.
	 */
	Object getAttribute(String pKey) throws JAXRException;
}
