package net.sf.csutils.core.model;

import java.util.Map;

import javax.xml.registry.JAXRException;


/**
 * Interface of a registry object type.
 */
public interface ROType {
	/**
	 * Returns the object types qualified name.
	 * @return The object types qualified name.
	 * @throws JAXRException The operation failed.
	 */
	QName getQName() throws JAXRException;

	/**
     * Returns the object types query name. In most cases, this is the same than
     * the qualified name. However, for builtin object types, it may differ.
     * For example, for "Organization" the query type is
     * "{http://namespaces.CentraSite.com/Schema/jaxr}organization".
     * @throws JAXRException The operation failed.
     * @return The object types query name.
     */
    QName getQueryName() throws JAXRException;

	/**
	 * Returns the map of all attributes.
	 * @throws JAXRException The operation failed.
	 * @return The map of attributes.
	 */
	Map<String,ROAttribute> getAttributes() throws JAXRException;

	/**
	 * Returns the attribute with the given name.
	 * @param pName The attribute name
     * @throws JAXRException The operation failed.
     * @return The attribute with the given name.
	 */
	ROAttribute getAttribute(String pName) throws JAXRException;
}
