package net.sf.csutils.core.model.impl;

import java.io.File;
import java.io.OutputStream;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBException;
import javax.xml.registry.JAXRException;
import javax.xml.stream.XMLEventWriter;
import javax.xml.stream.XMLStreamWriter;
import javax.xml.transform.Result;

import net.sf.csutils.core.model.QName;
import net.sf.csutils.core.model.ROAttribute;
import net.sf.csutils.core.model.ROClassification;
import net.sf.csutils.core.model.ROFile;
import net.sf.csutils.core.model.ROMetaModel;
import net.sf.csutils.core.model.RORelation;
import net.sf.csutils.core.model.ROSlot;
import net.sf.csutils.core.model.ROType;
import net.sf.csutils.core.model.vo.ObjectFactory;
import net.sf.csutils.core.model.vo.ROAttributeType;
import net.sf.csutils.core.model.vo.ROTypeReference;
import net.sf.csutils.core.utils.JaxbWriter;

import org.w3c.dom.Node;
import org.xml.sax.ContentHandler;


/**
 * The registry object model writer is used to serialize
 * registry meta models into XML.
 */
public class ROMetaModelWriter {
	private static final JaxbWriter<net.sf.csutils.core.model.vo.ROMetaModel> jaxbWriter =
		new JaxbWriter<net.sf.csutils.core.model.vo.ROMetaModel>(net.sf.csutils.core.model.vo.ROMetaModel.class,
			new javax.xml.namespace.QName("http://namespaces.csutils.sf.net/model", "roMetaModel"));
	private static final ObjectFactory of = new ObjectFactory();
	private static final Comparator<QName> typeComparator = new Comparator<QName>(){
		@Override
		public int compare(QName pQName1, QName pQName2) {
			final String localName1 = pQName1.getLocalPart();
			final String localName2 = pQName2.getLocalPart();
			int result = localName1.compareToIgnoreCase(localName2);
			if (result == 0) {
				final String uri1 = pQName1.getNamespaceURI();
				final String uri2 = pQName2.getNamespaceURI();
				result = uri1.compareToIgnoreCase(uri2);
			}
			return result;
		}
	};
	private static final Comparator<ROAttribute> attrComparator = new Comparator<ROAttribute>(){
		@Override
		public int compare(ROAttribute pAttr1, ROAttribute pAttr2) {
			ROAttribute.Type type1 = pAttr1.getType();
			ROAttribute.Type type2 = pAttr2.getType();
			final int result;
			switch (type1) {
				case relation:
					switch (type2) {
						case relation:
							result = 0;
							break;
						case classification:
						case file:
							result = -1;
							break;
						default:
							result = 1;
							break;
					}
					break;
				case classification:
					switch (type2) {
						case classification:
							result = 0;
							break;
						case file:
							result = -1;
							break;
						default:
							result = 1;
							break;
					}
					break;
				case file:
					switch (type2) {
						case file:
							result = 0;
							break;
						default:
							result = 1;
							break;
					}
					break;
				default:
					switch (type2) {
						case relation:
						case classification:
						case file:
							result = -1;
							break;
						default:
							result = 0;
							break;
					}
					break;
			}
			if (result == 0) {
				return pAttr1.getName().compareToIgnoreCase(pAttr2.getName());
			}
			return result;
		}
	};

	private void map(ROAttribute pAttr,
			final net.sf.csutils.core.model.vo.ROAttribute pBean) {
		pBean.setName(pAttr.getName());
		final int maxOccurs = pAttr.getMaxOccurs();
		if (maxOccurs != 1) {
			pBean.setMaxOccurs(maxOccurs == -1 ? "unbounded" : String.valueOf(maxOccurs));
		}
		final int minOccurs = pAttr.getMinOccurs();
		if (minOccurs != 1) {
			pBean.setMinOccurs(Integer.valueOf(minOccurs));
		}
	}

	protected net.sf.csutils.core.model.vo.ROSlot write(ROSlot pAttr) {
        final net.sf.csutils.core.model.vo.ROSlot bean = of.createROSlot();
        map(pAttr, bean);
        bean.setType(ROAttributeType.fromValue(pAttr.getType().name()));
        return bean;
    }

	/**
	 * Converts the given file attribute into a JAXB bean.
	 * @param pAttr The file attribute to convert.
	 * @return The created JAXB bean.
     * @throws JAXRException The operation failed.
     */
	protected net.sf.csutils.core.model.vo.ROFile write(ROFile pAttr) throws JAXRException {
        final net.sf.csutils.core.model.vo.ROFile bean = of.createROFile();
        map(pAttr, bean);
        bean.setAttributeKey(pAttr.getAttributeKey());
        return bean;
	}
	
    /**
     * Converts the given relationship attribute into a JAXB bean.
     * @param pAttr The relationship attribute to convert.
     * @return The created JAXB bean.
     * @throws JAXRException The operation failed.
     */
	protected net.sf.csutils.core.model.vo.RORelation write(RORelation pAttr) throws JAXRException {
		final net.sf.csutils.core.model.vo.RORelation bean = of.createRORelation();
		map(pAttr, bean);
		bean.setAssociationType(pAttr.getAssociationType());
		bean.setAttributeKey(pAttr.getAttributeKey());
		final List<ROType> targetTypes = pAttr.getTargetTypes();
		if (!targetTypes.isEmpty()) {
			final net.sf.csutils.core.model.vo.RORelation.TargetTypes targetTypesBean = of.createRORelationTargetTypes();
			bean.setTargetTypes(targetTypesBean);
			for (ROType targetType : targetTypes) {
				final ROTypeReference ref = of.createROTypeReference();
				targetTypesBean.getTargetType().add(ref);
				ref.setQName(targetType.getQName().toString());
				
			}
		}
		return bean;
	}

    /**
     * Converts the given classification attribute into a JAXB bean.
     * @param pAttr The classification attribute to convert.
     * @return The created JAXB bean.
     * @throws JAXRException The operation failed.
     */
	protected net.sf.csutils.core.model.vo.ROClassification write(ROClassification pAttr) {
	    final net.sf.csutils.core.model.vo.ROClassification bean = of.createROClassification();
	    map(pAttr, bean);
	    bean.setTaxonomy(pAttr.getTaxonomy());
	    bean.setAttributeKey(pAttr.getAttributeKey());
	    return bean;
	}

    /**
     * Converts the given registry object type into a JAXB bean.
     * @param pType The registry object type to convert.
     * @return The created JAXB bean.
     * @throws JAXRException  The operation failed.
     */
	protected net.sf.csutils.core.model.vo.ROType write(ROType pType) throws JAXRException {
		final net.sf.csutils.core.model.vo.ROType bean = of.createROType();
		final QName qName = pType.getQName();
		bean.setQName(qName.toString());
		final QName queryName = pType.getQueryName();
		if (queryName != null  &&  !queryName.equals(qName)) {
		    bean.setQueryName(queryName.toString());
		}
		final Map<String,ROAttribute> attrs = pType.getAttributes();
		if (!attrs.isEmpty()) {
			net.sf.csutils.core.model.vo.ROType.Attributes attrList = of.createROTypeAttributes();
			bean.setAttributes(attrList);
			final List<ROAttribute> sortedAttrs = new ArrayList<ROAttribute>(attrs.values());
			Collections.sort(sortedAttrs, attrComparator);
			for (ROAttribute attr : sortedAttrs) {
				final net.sf.csutils.core.model.vo.ROAttribute attrBean;
				switch (attr.getType()) {
					case relation:
						attrBean = write((RORelation) attr);
						break;
					case classification:
					    attrBean = write((ROClassification) attr);
					    break;
					case file:
					    attrBean = write((ROFile) attr);
					    break;
					default:
						attrBean = write((ROSlot) attr);
						break;
				}
				attrList.getAttributeOrRelationOrClassification().add(attrBean);
			}
		}
		return bean;
	}
	
	/**
	 * Converts the given meta model into a serializable JAXB bean.
	 * @param pModel The meta model to convert.
	 * @return The created JAXB bean.
	 * @throws JAXRException The operation failed.
	 */
	protected net.sf.csutils.core.model.vo.ROMetaModel write(net.sf.csutils.core.model.ROMetaModel pModel) throws JAXRException {
		final net.sf.csutils.core.model.vo.ROMetaModel bean = of.createROMetaModel();
		final Map<QName,ROType> types = pModel.getROTypes();
		final List<QName> qNames = new ArrayList<QName>(types.keySet());
		if (!qNames.isEmpty()) {
			final net.sf.csutils.core.model.vo.ROMetaModel.RoTypes typeList
				= of.createROMetaModelRoTypes();
			bean.setRoTypes(typeList);
			Collections.sort(qNames, typeComparator);
			for (QName qName : qNames) {
				final ROType type = types.get(qName);
				typeList.getRoType().add(write(type));
			}
		}
		return bean;
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link OutputStream}.
	 * @param pModel The meta model to convert.
	 * @param pStream The stream to write to.
	 * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, OutputStream pStream) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pStream);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link File}.
     * @param pModel The meta model to convert.
     * @param pFile The file to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, File pFile) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pFile);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link Writer}.
     * @param pModel The meta model to convert.
     * @param pWriter The writer to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, Writer pWriter) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pWriter);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link Result}.
     * @param pModel The meta model to convert.
     * @param pResult The transformer result to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, Result pResult) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pResult);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link Node}.
     * @param pModel The meta model to convert.
     * @param pNode The node to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, Node pNode) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pNode);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link XMLEventWriter}.
     * @param pModel The meta model to convert.
     * @param pWriter The StAX event writer to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, XMLEventWriter pWriter) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pWriter);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link XMLStreamWriter}.
     * @param pModel The meta model to convert.
     * @param pWriter The StAX stream writer to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(ROMetaModel pModel, XMLStreamWriter pWriter) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pWriter);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * {@link ContentHandler}.
     * @param pModel The meta model to convert.
     * @param pHandler The SAX content handler to write to.
     * @throws JAXRException The operation failed.
	 */
	public void write(net.sf.csutils.core.model.ROMetaModel pModel, ContentHandler pHandler) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pHandler);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}

	/**
	 * Converts the given meta model to XML by writing to the given
	 * output object.
     * @param pModel The meta model to convert.
     * @param pOutput The output object.
     * @throws JAXBException The operation failed.
	 */
	public void write(net.sf.csutils.core.model.ROMetaModel pModel, Object pOutput) throws JAXRException {
		try {
			jaxbWriter.write(write(pModel), pOutput);
		} catch (JAXBException e) {
			throw new JAXRException(e);
		}
	}
}
