package net.sf.csutils.core.query;

import java.util.Calendar;
import java.util.Collection;

import javax.xml.registry.JAXRException;


/**
 * Utility class for working with CSQL statements.
 */
public class StatementUtils {
    private StatementUtils() {
        // Private constructor, to prevent accidental instantiation.
    }

    public static void setParameter(CsqlStatement pStmt, int pIndex, Object pValue) throws JAXRException {
        if (pValue == null) {
            throw new IllegalArgumentException("A parameter value must not be null.");
        }
        if (pValue instanceof String) {
            pStmt.setString(pIndex, (String) pValue);
        } else if (pValue instanceof Boolean) {
            pStmt.setBoolean(pIndex, ((Boolean) pValue).booleanValue());
        } else if (pValue instanceof Byte) {
            pStmt.setByte(pIndex, ((Byte) pValue).byteValue());
        } else if (pValue instanceof Short) {
            pStmt.setShort(pIndex, ((Short) pValue).shortValue());
        } else if (pValue instanceof Integer) {
            pStmt.setInt(pIndex, ((Integer) pValue).intValue());
        } else if (pValue instanceof Long) {
            pStmt.setLong(pIndex, ((Long) pValue).longValue());
        } else if (pValue instanceof Float) {
            pStmt.setFloat(pIndex, ((Float) pValue).floatValue());
        } else if (pValue instanceof Double) {
            pStmt.setDouble(pIndex, ((Double) pValue).doubleValue());
        } else if (pValue instanceof Calendar) {
            pStmt.setDateTime(pIndex, (Calendar) pValue);
        } else {
            throw new IllegalArgumentException("Invalid parameter value: " + pValue.getClass().getName());
        }
    }
    
    public static void setParameters(CsqlStatement pStmt, Object... pParameters) throws JAXRException {
        for (int i = 0;  i < pParameters.length;  i++) {
            setParameter(pStmt, i, pParameters[i]);
        }
    }

    public static void setParameters(CsqlStatement pStmt, Collection<?> pParameters) throws JAXRException {
        int i = 0;
        for (Object o : pParameters) {
            setParameter(pStmt, i, o);
        }
    }
}
