// $ANTLR 2.7.7 (20060906): "csql.g" -> "CsqlBaseParser.java"$

//   $Id: hql.g 10163 2006-07-26 15:07:50Z steve.ebersole@jboss.com $

package net.sf.csutils.core.query.antlr;

import java.util.ArrayList;
import java.util.List;

import net.sf.csutils.core.query.tree.Identifier;
import net.sf.csutils.core.query.tree.NamespaceDeclarationSet;
import net.sf.csutils.core.query.tree.NodeFactory;
import net.sf.csutils.core.query.tree.ObjectType;
import net.sf.csutils.core.query.tree.SelectStatement;
import net.sf.csutils.core.query.tree.Statement;

import antlr.collections.AST;


import antlr.TokenBuffer;
import antlr.TokenStreamException;
import antlr.TokenStreamIOException;
import antlr.ANTLRException;
import antlr.LLkParser;
import antlr.Token;
import antlr.TokenStream;
import antlr.RecognitionException;
import antlr.NoViableAltException;
import antlr.MismatchedTokenException;
import antlr.SemanticException;
import antlr.ParserSharedInputState;
import antlr.collections.impl.BitSet;

/**
 * Hibernate Query Language Grammar
 * <br>
 * This grammar parses the query language for Hibernate (an Open Source, Object-Relational
 * mapping library).  A partial BNF grammar description is available for reference here:
 * http://www.hibernate.org/Documentation/HQLBNF
 *
 * Text from the original reference BNF is prefixed with '//##'.
 * @author Joshua Davis (pgmjsd@sourceforge.net)
 */
public class CsqlBaseParser extends antlr.LLkParser       implements CsqlTokenTypes
 {

    /** True if this is a filter query (allow no FROM clause). **/
	private boolean filter = false;
	private NodeFactory factory = new NodeFactory(){
        @Override
        public RecognitionException error(String pMessage) {
            try {
                final Token token = LT(0);
                return new RecognitionException(pMessage, getFilename(), token.getLine(), token.getColumn());
            } catch (TokenStreamException e) {
                return new RecognitionException(pMessage);
            }
        }
	};

	/**
	 * Sets the filter flag.
	 * @param f True for a filter query, false for a normal query.
	 */
	public void setFilter(boolean f) {
		filter = f;
	}

	/**
	 * Returns true if this is a filter query, false if not.
	 * @return true if this is a filter query, false if not.
	 */
	public boolean isFilter() {
		return filter;
	}

	/**
	 * This method is overriden in the sub class in order to provide the
	 * 'keyword as identifier' hack.
	 * @param token The token to retry as an identifier.
	 * @param ex The exception to throw if it cannot be retried as an identifier.
	 */
	public AST handleIdentifierError(Token token,RecognitionException ex) throws RecognitionException, TokenStreamException {
		// Base implementation: Just re-throw the exception.
		throw ex;
	}

    /**
     * This method looks ahead and converts . <token> into . IDENT when
     * appropriate.
     */
    public void handleDotIdent() throws TokenStreamException {
    }

	public void weakKeywords() throws TokenStreamException { }

protected CsqlBaseParser(TokenBuffer tokenBuf, int k) {
  super(tokenBuf,k);
  tokenNames = _tokenNames;
}

public CsqlBaseParser(TokenBuffer tokenBuf) {
  this(tokenBuf,3);
}

protected CsqlBaseParser(TokenStream lexer, int k) {
  super(lexer,k);
  tokenNames = _tokenNames;
}

public CsqlBaseParser(TokenStream lexer) {
  this(lexer,3);
}

public CsqlBaseParser(ParserSharedInputState state) {
  super(state,3);
  tokenNames = _tokenNames;
}

	public final Statement  statement() throws RecognitionException, TokenStreamException {
		Statement st = null;
		
		
		try {      // for error handling
			NamespaceDeclarationSet nds;
			nds=namespaceDeclarations();
			st=selectStatement(nds);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_0);
		}
		return st;
	}
	
	public final NamespaceDeclarationSet  namespaceDeclarations() throws RecognitionException, TokenStreamException {
		NamespaceDeclarationSet nds = null;
		
		
		try {      // for error handling
			String localPart; Identifier prefix;
			{
			_loop4:
			do {
				if ((LA(1)==DECLARE)) {
					match(DECLARE);
					match(NAMESPACE);
					prefix=identifier();
					match(EQ);
					localPart=stringConstant();
					nds = factory.namespaceDeclaration(nds, prefix.getPath(), localPart);
				}
				else {
					break _loop4;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_1);
		}
		return nds;
	}
	
	public final SelectStatement  selectStatement(
		NamespaceDeclarationSet nds
	) throws RecognitionException, TokenStreamException {
		SelectStatement st;
		
		st = factory.selectStatement(nds);
		
		try {      // for error handling
			queryRule(st);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_0);
		}
		return st;
	}
	
	public final Identifier  identifier() throws RecognitionException, TokenStreamException {
		Identifier id = null;
		
		
		try {      // for error handling
			final String s = LT(1).getText();
			match(IDENT);
			id = factory.identifier(s); /* TODO: return String value */
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_2);
		}
		return id;
	}
	
	public final String  stringConstant() throws RecognitionException, TokenStreamException {
		String s = null;
		
		
		try {      // for error handling
			String str = LT(1).getText();
			match(QUOTED_STRING);
			s = factory.stringConstant(str);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_3);
		}
		return s;
	}
	
	public final void queryRule(
		SelectStatement st
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			selectFrom(st);
			{
			switch ( LA(1)) {
			case WHERE:
			{
				Object o;
				o=whereClause();
				st.setWhereClause(o);
				break;
			}
			case EOF:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_0);
		}
	}
	
	public final void selectFrom(
		SelectStatement st
	) throws RecognitionException, TokenStreamException {
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case FROM:
			{
				fromClause(st);
				break;
			}
			case EOF:
			case WHERE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_4);
		}
	}
	
	public final Object  whereClause() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			match(WHERE);
			o=logicalExpression();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_0);
		}
		return o;
	}
	
	public final void fromClause(
		SelectStatement st
	) throws RecognitionException, TokenStreamException {
		
		ObjectType ot;
		
		try {      // for error handling
			match(FROM);
			weakKeywords();
			ot=fromRange();
			factory.objectType(st, ot);
			{
			_loop12:
			do {
				if ((LA(1)==JOIN)) {
					ot=fromJoin();
					factory.objectType(st, ot);
				}
				else {
					break _loop12;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_4);
		}
	}
	
	public final ObjectType  fromRange() throws RecognitionException, TokenStreamException {
		ObjectType ot = null;
		
		
		try {      // for error handling
			ot=fromClassOrOuterQueryPath();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_5);
		}
		return ot;
	}
	
	public final ObjectType  fromJoin() throws RecognitionException, TokenStreamException {
		ObjectType ot = null;
		
		Identifier id; String a;
		
		try {      // for error handling
			match(JOIN);
			id=path();
			a=asAlias();
			ot = factory.objectType(id.getPath(), a);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_5);
		}
		return ot;
	}
	
	public final Identifier  path() throws RecognitionException, TokenStreamException {
		Identifier id = null;
		
		
		try {      // for error handling
			id=identifier();
			{
			_loop75:
			do {
				if ((LA(1)==DOT)) {
					match(DOT);
					weakKeywords();
					Identifier id2;
					id2=identifier();
					id = factory.identifier(id, id2);
				}
				else {
					break _loop75;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_6);
		}
		return id;
	}
	
	public final String  asAlias() throws RecognitionException, TokenStreamException {
		String s = null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case AS:
			{
				match(AS);
				break;
			}
			case IDENT:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			s=alias();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_5);
		}
		return s;
	}
	
	public final ObjectType  fromClassOrOuterQueryPath() throws RecognitionException, TokenStreamException {
		ObjectType ot = null;
		
		Identifier id; String a;
		
		try {      // for error handling
			id=objectTypePath();
			weakKeywords();
			a=asAlias();
			ot = factory.objectType(id.getPath(), a);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_5);
		}
		return ot;
	}
	
	public final Identifier  objectTypePath() throws RecognitionException, TokenStreamException {
		Identifier id = null;
		
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case STAR:
			{
				match(STAR);
				id = factory.allTypes();
				break;
			}
			case IDENT:
			{
				Identifier prefix = null;
				{
				if ((LA(1)==IDENT) && (LA(2)==COLON)) {
					prefix=identifier();
					match(COLON);
				}
				else if ((LA(1)==IDENT) && (LA(2)==AS||LA(2)==IDENT)) {
				}
				else {
					throw new NoViableAltException(LT(1), getFilename());
				}
				
				}
				id=identifier();
				if (prefix != null) { id = factory.identifier(prefix.getPath() + ":" + id.getPath()); }
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_6);
		}
		return id;
	}
	
	public final String  alias() throws RecognitionException, TokenStreamException {
		String s = null;
		
		Identifier id;
		
		try {      // for error handling
			id=identifier();
			s = id.getPath();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_5);
		}
		return s;
	}
	
	public final Object  logicalExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			o=expression();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_0);
		}
		return o;
	}
	
	public final Object  expression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			o=logicalOrExpression();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_7);
		}
		return o;
	}
	
	public final Object  logicalOrExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			o=logicalAndExpression();
			{
			_loop24:
			do {
				if ((LA(1)==OR)) {
					match(OR);
					Object o2;
					o2=logicalAndExpression();
					o = factory.or(o, o2);
				}
				else {
					break _loop24;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_7);
		}
		return o;
	}
	
	public final Object  logicalAndExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			o=negatedExpression();
			{
			_loop27:
			do {
				if ((LA(1)==AND)) {
					match(AND);
					Object o2;
					o2=negatedExpression();
					o = factory.and(o, o2);
				}
				else {
					break _loop27;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_8);
		}
		return o;
	}
	
	public final Object  negatedExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		weakKeywords();
		
		try {      // for error handling
			switch ( LA(1)) {
			case NOT:
			{
				match(NOT);
				o=negatedExpression();
				o = factory.negate(o);
				break;
			}
			case FALSE:
			case TRUE:
			case NUM_DOUBLE:
			case NUM_FLOAT:
			case NUM_LONG:
			case OPEN:
			case PLUS:
			case MINUS:
			case COLON:
			case PARAM:
			case NUM_INT:
			case QUOTED_STRING:
			case IDENT:
			{
				o=equalityExpression();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_9);
		}
		return o;
	}
	
	public final Object  equalityExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		Object o2;
		
		try {      // for error handling
			o=relationalExpression();
			{
			_loop35:
			do {
				switch ( LA(1)) {
				case EQ:
				{
					{
					match(EQ);
					o2=relationalExpression();
					o = factory.eq(o, o2);
					}
					break;
				}
				case NE:
				{
					{
					match(NE);
					o2=relationalExpression();
					o = factory.ne(o, o2);
					}
					break;
				}
				case SQL_NE:
				{
					{
					match(SQL_NE);
					o2=relationalExpression();
					o = factory.ne(o, o2);
					}
					break;
				}
				case IS:
				{
					match(IS);
					{
					switch ( LA(1)) {
					case NULL:
					{
						match(NULL);
						o = factory.isNull(o);
						break;
					}
					case EMPTY:
					{
						match(EMPTY);
						o = factory.isEmpty(o);
						break;
					}
					default:
					{
						throw new NoViableAltException(LT(1), getFilename());
					}
					}
					}
					break;
				}
				default:
				{
					break _loop35;
				}
				}
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_9);
		}
		return o;
	}
	
	public final Object  relationalExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		Object o2;
		
		try {      // for error handling
			o=concatenation();
			{
			_loop38:
			do {
				switch ( LA(1)) {
				case LT:
				{
					match(LT);
					o2=additiveExpression();
					o = factory.lt(o, o2);
					break;
				}
				case GT:
				{
					match(GT);
					o2=additiveExpression();
					o = factory.gt(o, o2);
					break;
				}
				case LE:
				{
					match(LE);
					o2=additiveExpression();
					o = factory.le(o, o2);
					break;
				}
				case GE:
				{
					match(GE);
					o2=additiveExpression();
					o = factory.ge(o, o2);
					break;
				}
				case LIKE:
				{
					match(LIKE);
					o2=additiveExpression();
					o = factory.like(o, o2);
					break;
				}
				case IN:
				{
					match(IN);
					o2=inList();
					o = factory.in(o, (List<?>) o2, false);
					break;
				}
				default:
					if ((LA(1)==NOT) && (LA(2)==LIKE)) {
						match(NOT);
						match(LIKE);
						o2=additiveExpression();
						o = factory.notLike(o, o2);
					}
					else if ((LA(1)==NOT) && (LA(2)==IN)) {
						match(NOT);
						match(IN);
						o2=inList();
						o = factory.in(o, (List<?>) o2, true);
					}
				else {
					break _loop38;
				}
				}
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_10);
		}
		return o;
	}
	
	public final Object  concatenation() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		Object o2;
		
		try {      // for error handling
			o=additiveExpression();
			{
			_loop44:
			do {
				if ((LA(1)==CONCAT)) {
					match(CONCAT);
					o2=additiveExpression();
					o = factory.concatenate(o, o2);
				}
				else {
					break _loop44;
				}
				
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_11);
		}
		return o;
	}
	
	public final Object  additiveExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		Object o2;
		
		try {      // for error handling
			o=multiplyExpression();
			{
			_loop49:
			do {
				switch ( LA(1)) {
				case PLUS:
				{
					{
					match(PLUS);
					o2=multiplyExpression();
					o = factory.addExpression(o, o2);
					}
					break;
				}
				case MINUS:
				{
					{
					match(MINUS);
					o2=multiplyExpression();
					o = factory.minusExpression(o, o2);
					}
					break;
				}
				default:
				{
					break _loop49;
				}
				}
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_12);
		}
		return o;
	}
	
	public final List  inList() throws RecognitionException, TokenStreamException {
		List list = new ArrayList();
		
		Object o;
		
		try {      // for error handling
			match(OPEN);
			o=expression();
			list.add(o);
			{
			_loop41:
			do {
				if ((LA(1)==COMMA)) {
					match(COMMA);
					o=expression();
					list.add(o);
				}
				else {
					break _loop41;
				}
				
			} while (true);
			}
			match(CLOSE);
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_11);
		}
		return list;
	}
	
	public final Object  multiplyExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		Object o2;
		
		try {      // for error handling
			o=unaryExpression();
			{
			_loop54:
			do {
				switch ( LA(1)) {
				case STAR:
				{
					{
					match(STAR);
					o2=unaryExpression();
					o = factory.multiplyExpression(o, o2);
					}
					break;
				}
				case DIV:
				{
					{
					match(DIV);
					o2=unaryExpression();
					o = factory.divideExpression(o, o2);
					}
					break;
				}
				default:
				{
					break _loop54;
				}
				}
			} while (true);
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_13);
		}
		return o;
	}
	
	public final Object  unaryExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case MINUS:
			{
				match(MINUS);
				o=unaryExpression();
				o = factory.unaryMinus(o);
				break;
			}
			case PLUS:
			{
				match(PLUS);
				o=unaryExpression();
				break;
			}
			case FALSE:
			case TRUE:
			case NUM_DOUBLE:
			case NUM_FLOAT:
			case NUM_LONG:
			case OPEN:
			case COLON:
			case PARAM:
			case NUM_INT:
			case QUOTED_STRING:
			case IDENT:
			{
				o=atom();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_14);
		}
		return o;
	}
	
	public final Object  atom() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			o=primaryExpression();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_14);
		}
		return o;
	}
	
	public final Object  primaryExpression() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			switch ( LA(1)) {
			case IDENT:
			{
				o=identPrimary();
				break;
			}
			case FALSE:
			case TRUE:
			case NUM_DOUBLE:
			case NUM_FLOAT:
			case NUM_LONG:
			case NUM_INT:
			case QUOTED_STRING:
			{
				o=constant();
				break;
			}
			case COLON:
			{
				match(COLON);
				Identifier id;
				id=identifier();
				o = factory.namedParameter(id.getPath());
				break;
			}
			case OPEN:
			{
				match(OPEN);
				{
				o=expressionOrVector();
				}
				match(CLOSE);
				break;
			}
			case PARAM:
			{
				match(PARAM);
				String s = null;
				{
				switch ( LA(1)) {
				case NUM_INT:
				{
					match(NUM_INT);
					break;
				}
				case EOF:
				case AND:
				case IN:
				case IS:
				case LIKE:
				case NOT:
				case OR:
				case EQ:
				case NE:
				case SQL_NE:
				case LT:
				case GT:
				case LE:
				case GE:
				case COMMA:
				case CLOSE:
				case CONCAT:
				case PLUS:
				case MINUS:
				case STAR:
				case DIV:
				{
					break;
				}
				default:
				{
					throw new NoViableAltException(LT(1), getFilename());
				}
				}
				}
				o = s == null ? factory.parameter() : factory.numberedParameter(s);
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_14);
		}
		return o;
	}
	
	public final Object  identPrimary() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		Identifier id = null;
		
		try {      // for error handling
			id=identifier();
			handleDotIdent();
			{
			_loop63:
			do {
				if ((LA(1)==DOT)) {
					match(DOT);
					Identifier id2;
					id2=identifier();
					id = factory.identifier(id, id2);
				}
				else {
					break _loop63;
				}
				
			} while (true);
			}
			{
			switch ( LA(1)) {
			case OPEN:
			{
				{
				match(OPEN);
				List<Object> args;
				args=exprList();
				o = factory.methodCall(id, args);
				match(CLOSE);
				}
				break;
			}
			case EOF:
			case AND:
			case IN:
			case IS:
			case LIKE:
			case NOT:
			case OR:
			case EQ:
			case NE:
			case SQL_NE:
			case LT:
			case GT:
			case LE:
			case GE:
			case COMMA:
			case CLOSE:
			case CONCAT:
			case PLUS:
			case MINUS:
			case STAR:
			case DIV:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
			if (o == null) { o = id; }
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_14);
		}
		return o;
	}
	
	public final Object  constant() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			String s = LT(1).getText();
			{
			switch ( LA(1)) {
			case NUM_INT:
			{
				match(NUM_INT);
				o = factory.integerConstant(s);
				break;
			}
			case NUM_FLOAT:
			{
				match(NUM_FLOAT);
				o = factory.floatConstant(s);
				break;
			}
			case NUM_LONG:
			{
				match(NUM_LONG);
				o = factory.longConstant(s);
				break;
			}
			case NUM_DOUBLE:
			{
				match(NUM_DOUBLE);
				o = factory.doubleConstant(s);
				break;
			}
			case QUOTED_STRING:
			{
				o=stringConstant();
				break;
			}
			case TRUE:
			{
				match(TRUE);
				o = factory.trueConstant();
				break;
			}
			case FALSE:
			{
				match(FALSE);
				o = factory.falseConstant();
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_14);
		}
		return o;
	}
	
	public final Object  expressionOrVector() throws RecognitionException, TokenStreamException {
		Object o = null;
		
		
		try {      // for error handling
			o=expression();
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_15);
		}
		return o;
	}
	
	public final List<Object>  exprList() throws RecognitionException, TokenStreamException {
		List<Object> list = new ArrayList<Object>();
		
		Object o;
		
		try {      // for error handling
			{
			switch ( LA(1)) {
			case FALSE:
			case NOT:
			case TRUE:
			case NUM_DOUBLE:
			case NUM_FLOAT:
			case NUM_LONG:
			case OPEN:
			case PLUS:
			case MINUS:
			case COLON:
			case PARAM:
			case NUM_INT:
			case QUOTED_STRING:
			case IDENT:
			{
				o=expression();
				list.add(o);
				{
				_loop69:
				do {
					if ((LA(1)==COMMA)) {
						match(COMMA);
						o=expression();
						list.add(o);
					}
					else {
						break _loop69;
					}
					
				} while (true);
				}
				break;
			}
			case CLOSE:
			{
				break;
			}
			default:
			{
				throw new NoViableAltException(LT(1), getFilename());
			}
			}
			}
		}
		catch (RecognitionException ex) {
			reportError(ex);
			recover(ex,_tokenSet_15);
		}
		return list;
	}
	
	
	public static final String[] _tokenNames = {
		"<0>",
		"EOF",
		"<2>",
		"NULL_TREE_LOOKAHEAD",
		"\"all\"",
		"\"any\"",
		"\"and\"",
		"\"as\"",
		"\"asc\"",
		"\"avg\"",
		"\"between\"",
		"\"class\"",
		"\"count\"",
		"\"declare\"",
		"\"delete\"",
		"\"desc\"",
		"DOT",
		"\"distinct\"",
		"\"elements\"",
		"\"escape\"",
		"\"exists\"",
		"\"false\"",
		"\"fetch\"",
		"\"from\"",
		"\"full\"",
		"\"group\"",
		"\"having\"",
		"\"in\"",
		"\"indices\"",
		"\"inner\"",
		"\"insert\"",
		"\"into\"",
		"\"is\"",
		"\"join\"",
		"\"left\"",
		"\"like\"",
		"\"max\"",
		"\"min\"",
		"\"namespace\"",
		"\"new\"",
		"\"not\"",
		"\"null\"",
		"\"or\"",
		"\"order\"",
		"\"outer\"",
		"\"properties\"",
		"\"right\"",
		"\"select\"",
		"\"set\"",
		"\"some\"",
		"\"sum\"",
		"\"true\"",
		"\"union\"",
		"\"update\"",
		"\"versioned\"",
		"\"where\"",
		"\"case\"",
		"\"end\"",
		"\"else\"",
		"\"then\"",
		"\"when\"",
		"\"on\"",
		"\"with\"",
		"\"both\"",
		"\"empty\"",
		"\"leading\"",
		"\"member\"",
		"\"object\"",
		"\"of\"",
		"\"trailing\"",
		"AGGREGATE",
		"ALIAS",
		"CONSTRUCTOR",
		"CASE2",
		"EXPR_LIST",
		"FILTER_ENTITY",
		"IN_LIST",
		"INDEX_OP",
		"IS_NOT_NULL",
		"IS_NULL",
		"METHOD_CALL",
		"NOT_BETWEEN",
		"NOT_IN",
		"NOT_LIKE",
		"ORDER_ELEMENT",
		"QUERY",
		"RANGE",
		"ROW_STAR",
		"SELECT_FROM",
		"UNARY_MINUS",
		"UNARY_PLUS",
		"VECTOR_EXPR",
		"WEIRD_IDENT",
		"CONSTANT",
		"NUM_DOUBLE",
		"NUM_FLOAT",
		"NUM_LONG",
		"JAVA_CONSTANT",
		"EQ",
		"NE",
		"SQL_NE",
		"LT",
		"GT",
		"LE",
		"GE",
		"OPEN",
		"COMMA",
		"CLOSE",
		"CONCAT",
		"PLUS",
		"MINUS",
		"STAR",
		"DIV",
		"COLON",
		"PARAM",
		"NUM_INT",
		"QUOTED_STRING",
		"IDENT",
		"OPEN_BRACKET",
		"CLOSE_BRACKET",
		"ID_START_LETTER",
		"ID_LETTER",
		"ESCqs",
		"WS",
		"HEX_DIGIT",
		"EXPONENT",
		"FLOAT_SUFFIX"
	};
	
	private static final long[] mk_tokenSet_0() {
		long[] data = { 2L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_0 = new BitSet(mk_tokenSet_0());
	private static final long[] mk_tokenSet_1() {
		long[] data = { 36028797027352578L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_1 = new BitSet(mk_tokenSet_1());
	private static final long[] mk_tokenSet_2() {
		long[] data = { 36034341956026562L, 10133081981714432L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_2 = new BitSet(mk_tokenSet_2());
	private static final long[] mk_tokenSet_3() {
		long[] data = { 36034333374423106L, 560733750296576L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_3 = new BitSet(mk_tokenSet_3());
	private static final long[] mk_tokenSet_4() {
		long[] data = { 36028797018963970L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_4 = new BitSet(mk_tokenSet_4());
	private static final long[] mk_tokenSet_5() {
		long[] data = { 36028805608898562L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_5 = new BitSet(mk_tokenSet_5());
	private static final long[] mk_tokenSet_6() {
		long[] data = { 128L, 9007199254740992L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_6 = new BitSet(mk_tokenSet_6());
	private static final long[] mk_tokenSet_7() {
		long[] data = { 2L, 13194139533312L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_7 = new BitSet(mk_tokenSet_7());
	private static final long[] mk_tokenSet_8() {
		long[] data = { 4398046511106L, 13194139533312L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_8 = new BitSet(mk_tokenSet_8());
	private static final long[] mk_tokenSet_9() {
		long[] data = { 4398046511170L, 13194139533312L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_9 = new BitSet(mk_tokenSet_9());
	private static final long[] mk_tokenSet_10() {
		long[] data = { 4402341478466L, 13314398617600L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_10 = new BitSet(mk_tokenSet_10());
	private static final long[] mk_tokenSet_11() {
		long[] data = { 5536347062338L, 15375982919680L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_11 = new BitSet(mk_tokenSet_11());
	private static final long[] mk_tokenSet_12() {
		long[] data = { 5536347062338L, 32968168964096L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_12 = new BitSet(mk_tokenSet_12());
	private static final long[] mk_tokenSet_13() {
		long[] data = { 5536347062338L, 138521285230592L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_13 = new BitSet(mk_tokenSet_13());
	private static final long[] mk_tokenSet_14() {
		long[] data = { 5536347062338L, 560733750296576L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_14 = new BitSet(mk_tokenSet_14());
	private static final long[] mk_tokenSet_15() {
		long[] data = { 0L, 8796093022208L, 0L, 0L};
		return data;
	}
	public static final BitSet _tokenSet_15 = new BitSet(mk_tokenSet_15());
	
	}
