package net.sf.csutils.core.query.impl;

import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.List;

import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.RegistryObject;

import net.sf.csutils.core.query.CsqlStatement;
import net.sf.csutils.core.registry.RegistryFacade;

import org.apache.labs.jaxmas.registry.infomodel.ConnectionImpl;
import org.apache.labs.jaxmas.registry.sql.RegistryObjectLoader;
import org.apache.labs.jaxmas.registry.sql.StmtUser;
import org.apache.log4j.Logger;


public class SqlCsqlStatement implements CsqlStatement {
    private static final Logger log = Logger.getLogger(SqlCsqlStatement.class);
    private final RegistryObjectLoader rol;
    private final RegistryFacade facade;
    private final PreparedStatement stmt;
    private final int parameterOffset;

    public SqlCsqlStatement(RegistryFacade pFacade, RegistryObjectLoader pRol) throws JAXRException {
        rol = pRol;
        facade = pFacade;
        if (pFacade == null) {
            stmt = null;
            parameterOffset = 0;
        } else {
            final String query = pRol.getQuery();
            final Object[] parameters = pRol.getParameters();
            parameterOffset = parameters.length+1;

            final StringBuilder sb = new StringBuilder();
            sb.append("prepare: ");
            sb.append(query);
            for (int i = 0;  i < parameters.length;  i++) {
                sb.append(", ");
                sb.append(parameters[i]);
            }
            log.debug(sb);
            
            try {
                stmt = ((ConnectionImpl) pFacade.getConnection()).getConnection().prepareStatement(query);
                StmtUser.setParams(stmt, parameters);
            } catch (SQLException e) {
                throw new JAXRException(e);
            }
        }
    }

    @Override
    public SqlCsqlStatement clone(RegistryFacade pFacade) throws JAXRException {
        return new SqlCsqlStatement(pFacade, rol);
    }
    
    @Override
    public List<RegistryObject[]> executeArrayQuery() throws JAXRException {
        return rol.getResultArrayList(facade.getConnection().getRegistryService(), stmt);
    }

    @Override
    public List<RegistryObject> executeQuery() throws JAXRException {
        return rol.getResultList(facade.getConnection().getRegistryService(), stmt);
    }

    @Override
    public void setBoolean(int pNum, boolean pValue) throws JAXRException {
        try {
            stmt.setBoolean(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setBoolean(String pName, boolean pValue) {
        // TODO Auto-generated method stub
    }

    @Override
    public void setByte(int pNum, byte pValue) throws JAXRException {
        try {
            stmt.setByte(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setByte(String pName, byte pValue) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setShort(int pNum, short pValue) throws JAXRException {
        try {
            stmt.setShort(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setShort(String pName, short pValue) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setInt(int pNum, int pValue) throws JAXRException {
        try {
            stmt.setInt(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setInt(String pName, int pValue) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setLong(int pNum, long pValue) throws JAXRException {
        try {
            stmt.setLong(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setLong(String pName, long pValue) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setFloat(int pNum, float pValue) throws JAXRException {
        try {
            stmt.setFloat(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setFloat(String pName, float pValue) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setDouble(int pNum, double pValue) throws JAXRException {
        try {
            stmt.setDouble(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setDouble(String pName, double pValue) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setDateTime(int pNum, Calendar pCalendar) throws JAXRException {
        try {
            stmt.setTimestamp(pNum+parameterOffset, new Timestamp(pCalendar.getTimeInMillis()));
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setDateTime(String pName, Calendar pCalendar) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setDate(int pNum, Calendar pCalendar) throws JAXRException {
        try {
            stmt.setDate(pNum+parameterOffset, new Date(pCalendar.getTimeInMillis()));
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setDate(String pName, Calendar pCalendar) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setTime(int pNum, Calendar pCalendar) throws JAXRException {
        try {
            stmt.setTime(pNum+parameterOffset, new Time(pCalendar.getTimeInMillis()));
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setTime(String pName, Calendar pCalendar) {
        // TODO Auto-generated method stub
        
    }

    @Override
    public void setString(int pNum, String pValue) throws JAXRException {
        try {
            stmt.setString(pNum+parameterOffset, pValue);
        } catch (SQLException e) {
            throw new JAXRException(e);
        }
    }

    @Override
    public void setString(String pName, String pValue) {
        // TODO Auto-generated method stub
        
    }

    public String getSqlQuery() {
        return rol.getQuery();
    }
}
