package net.sf.csutils.core.query.tree;

import java.util.List;


/**
 * Representation of a method call in the tree.
 */
public class MethodCall {
	private final Identifier id;
	private final String methodName;
	private final List<Object> args;

	MethodCall(Identifier pId, String pMethodName, List<Object> pArgs) {
		id = pId;
		methodName = pMethodName;
		args = pArgs;
	}

	/**
	 * Returns the item, on which the method is invoked.
	 * @return The item, on which the method is invoked.
	 */
	public Identifier getItem() {
		return id;
	}

	/**
	 * Returns the method name.
	 * @return The method name.
	 */
	public String getMethodName() {
		return methodName;
	}
	
	/**
	 * Returns the argument list.
	 * @return The argument list.
	 */
	public List<Object> getArgs() {
		return args;
	}
}
