package net.sf.csutils.core.registry;

import javax.xml.registry.JAXRException;

import net.sf.csutils.core.model.QName;
import net.sf.csutils.core.model.ROMetaModel;
import net.sf.csutils.core.model.ROType;


/**
 * The meta model accessor is used to read, or modify the registry meta
 * model. Note, that meta model operations may enforce to reread the
 * meta model. In other words, they are likely to be a source for
 * problems with performance. Use this accessor with care!
 */
public interface ROMetaModelAccessor {
    /**
     * Returns the accessors registry facade.
     * @return The registry facade to use.
     */
    ModelDrivenRegistryFacade getRegistryFacade();

    /**
     * Returns the meta model.
     * @return The meta model to use.
     */
    ROMetaModel getMetaModel();

    /**
     * Removes the given object type from the registry. This includes
     * removal of instances, if there are any.
     * @param pQName Qualified name of the registry object type to remove.
     * @throws JAXRException The operation failed.
     */
    void remove(QName pQName) throws JAXRException;

    /**
     * Creates the given {@link ROType registry object type} in
     * the registry.
     * @param pType Description of the type to create.
     * @return An instance of {@link ROType}, which matches the type that has
     *   just been created.
     * @throws JAXRException The operation failed.
     */
    ROType create(ROType pType) throws JAXRException;
}
