package net.sf.csutils.core.registry;

import net.sf.csutils.core.registry.centrasite.CentraSiteRegistryInfo;
import net.sf.csutils.core.registry.impl.InnerModelAccessor;
import net.sf.csutils.core.registry.impl.MetaModelAccessor;


/**
 * This class allows to adapt between various type of registries.
 */
public abstract class RegistryInfo {
	private static RegistryInfo theInstance = CentraSiteRegistryInfo.getInstance();

	/**
	 * Returns the default instance.
	 * @return The default registry information object.
	 */
	public static RegistryInfo getInstance() {
		return theInstance;
	}

	/**
	 * Sets the default instance.
     * @param pInstance The new default registry information object.
	 */
	public static void setInstance(RegistryInfo pInstance) {
		theInstance = pInstance;
	}

	/**
	 * Returns the key of the object types classification scheme.
	 * @return The classification schemes key.
	 */
	public abstract String getObjectTypesKey();

	/**
	 * Returns the name of the object types classification scheme.
     * @return The classification schemes name.
	 */
	public abstract String getObjectTypesName();

	/**
	 * Returns the key of the association types classification scheme.
     * @return The classification schemes key.
	 */
	public abstract String getAssociationTypesKey();

	/**
	 * Returns the name of the association types classification scheme.
     * @return The classification schemes name.
	 */
	public abstract String getAssociationTypesName();

	/**
	 * Returns the {@link MetaModelAccessor}.
	 * @return The inner meta model accessor to use.
	 */
	public abstract MetaModelAccessor getMetaModelAccessor();

    /**
     * Returns the {@link InnerModelAccessor}.
     * @return The inner model accessor to use.
     */
    public abstract InnerModelAccessor getModelAccessor();
}
