package net.sf.csutils.core.registry;

import javax.xml.registry.Connection;
import javax.xml.registry.JAXRException;

import net.sf.csutils.core.model.ROMetaModel;
import net.sf.csutils.core.registry.centrasite.CentraSiteRegistryInfo;
import net.sf.csutils.core.registry.impl.ROMetaModelAccessorImpl;
import net.sf.csutils.core.registry.impl.ROModelAccessorImpl;
import net.sf.csutils.core.registry.jaxmas.JaxMasRegistryInfo;


/**
 * Implementation of {@link ModelDrivenRegistryFacade}.
 */
public class SimpleModelDrivenRegistryFacade extends SimpleRegistryFacade implements ModelDrivenRegistryFacade {
    private final ROMetaModelAccessor metaModelAccessor;
    private final ROModelAccessor modelAccessor;
    
    public SimpleModelDrivenRegistryFacade(Connection pConnection) throws JAXRException {
        super(getRegistryInfo(pConnection), pConnection);
        metaModelAccessor = new ROMetaModelAccessorImpl(this, getRegistryInfo().getMetaModelAccessor());
        modelAccessor = new ROModelAccessorImpl(this, metaModelAccessor, getRegistryInfo().getModelAccessor());
    }

    /**
     * Returns a proper instance of {@link RegistryInfo}, which is suitable for the given
     * JAXR connection.
     * @param pConnection The connection to inspect
     * @return An instance of {@link RegistryInfo}, which is suitable for the given JAXR connection.
     */
    public static RegistryInfo getRegistryInfo(Connection pConnection) {
		return pConnection.getClass().getName().startsWith("org.apache.labs") ? JaxMasRegistryInfo.getInstance() : CentraSiteRegistryInfo.getInstance();
	}

    public SimpleModelDrivenRegistryFacade(RegistryInfo pRegistryInfo,
            Connection pConnection) throws JAXRException {
        super(pRegistryInfo, pConnection);
        metaModelAccessor = new ROMetaModelAccessorImpl(this, getRegistryInfo().getMetaModelAccessor());
        modelAccessor = new ROModelAccessorImpl(this, metaModelAccessor, getRegistryInfo().getModelAccessor());
    }

    @Override
    public ROMetaModel getMetaModel() {
        return metaModelAccessor.getMetaModel();
    }

    @Override
    public ROMetaModelAccessor getMetaModelAccessor() {
        return metaModelAccessor;
    }

    @Override
    public ROModelAccessor getModelAccessor() {
        return modelAccessor;
    }
}
