package net.sf.csutils.core.registry.centrasite;

import java.util.HashSet;
import java.util.Set;

import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.JAXRException;
import javax.xml.registry.LifeCycleManager;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.RegistryObject;

import net.sf.csutils.core.registry.RegistryFacade;
import net.sf.csutils.core.registry.impl.AbstractInnerModelAccessor;
import net.sf.csutils.core.registry.impl.InnerModelAccessor;


/**
 * Implementation of {@link InnerModelAccessor} for JaxMas.
 */
public class CentraSiteInnerModelAccessor extends AbstractInnerModelAccessor {
    private static final CentraSiteInnerModelAccessor theInstance = new CentraSiteInnerModelAccessor();
    private static final Set<String> BUILTIN_TYPES = new HashSet<String>();
    static {
        BUILTIN_TYPES.add(LifeCycleManager.ASSOCIATION);
        BUILTIN_TYPES.add(LifeCycleManager.AUDITABLE_EVENT);
        BUILTIN_TYPES.add("CentraSiteFilter");
        BUILTIN_TYPES.add("CentraSiteOperation");
        BUILTIN_TYPES.add(LifeCycleManager.CLASSIFICATION);
        BUILTIN_TYPES.add(LifeCycleManager.CLASSIFICATION_SCHEME);
        BUILTIN_TYPES.add(LifeCycleManager.CONCEPT);
        BUILTIN_TYPES.add(LifeCycleManager.EXTERNAL_IDENTIFIER);
        BUILTIN_TYPES.add(LifeCycleManager.EXTERNAL_LINK);
        BUILTIN_TYPES.add(LifeCycleManager.EXTRINSIC_OBJECT);
        BUILTIN_TYPES.add(LifeCycleManager.ORGANIZATION);
        BUILTIN_TYPES.add(LifeCycleManager.REGISTRY_PACKAGE);
        BUILTIN_TYPES.add(LifeCycleManager.SERVICE);
        BUILTIN_TYPES.add(LifeCycleManager.SERVICE_BINDING);
        BUILTIN_TYPES.add(LifeCycleManager.SPECIFICATION_LINK);
        BUILTIN_TYPES.add(LifeCycleManager.USER);
    }

    private CentraSiteInnerModelAccessor() {
        // Private constructor, to ensure singleton pattern.
    }

    /**
     * Returns the singleton instance
     * @return The singleton instance.
     */
    public static CentraSiteInnerModelAccessor getInstance() {
        return theInstance;
    }

    @SuppressWarnings("unchecked")
    @Override
    public <RO extends RegistryObject> RO createInstance(
            RegistryFacade pFacade, String pType) throws JAXRException {
        final BusinessLifeCycleManager blcm = pFacade.getBusinessLifeCycleManager();
        final RegistryObject ro;
        if (BUILTIN_TYPES.contains(pType)) {
            ro = (RegistryObject) blcm.createObject(pType);
        } else {
            ro = (RegistryObject) blcm.createObject(LifeCycleManager.REGISTRY_ENTRY);
            final Concept objectType = pFacade.findObjectType(pType);
            if (objectType == null) {
                throw new JAXRException("No such object type: " + pType);
            }
            ro.addClassification(blcm.createClassification(objectType));
        }
        return (RO) ro;
    }
}
