package net.sf.csutils.core.registry.centrasite;

import net.sf.csutils.core.registry.RegistryInfo;
import net.sf.csutils.core.registry.impl.InnerModelAccessor;
import net.sf.csutils.core.registry.impl.MetaModelAccessor;


/**
 * CentraSite specific implementation of {@link RegistryInfo}.
 * This is the default.
 */
public class CentraSiteRegistryInfo extends RegistryInfo {
    /**
     * The namespace "http://namespaces.CentraSite.com/Schema/jaxr", which is the
     * namespace of the builtin object types.
     */
    public static final String NS_SCHEMA_JAXR = "http://namespaces.CentraSite.com/Schema/jaxr";

    /**
     * The namespace "http://namespaces.CentraSite.com/Schema", which is the
     * namespace of other CentraSite related information, like the attribute keys.
     */
    public static final String NS_SCHEMA = "http://namespaces.CentraSite.com/Schema";

    /**
	 * The key of the classification scheme {@link CentraSiteRegistryInfo#OBJECT_TYPES_NAME}.
	 * Object type concepts are direct children of this classification
	 * scheme.
	 */
	public static final String OBJECT_TYPES_KEY = "uddi:474e1904-69ad-9a6b-11c2-fc8e41c64350";

	/**
	 * The name of the classification scheme of object types.
	 * Object type concepts are direct children of this classification
	 * scheme.
	 */
	public static final String OBJECT_TYPES_NAME = "ObjectType";

	/**
	 * The key of the classification scheme {@link CentraSiteRegistryInfo#ASSOCIATION_TYPES_NAME}.
	 * Association type concepts are direct children of this classification
	 * scheme.
	 */
	public static final String ASSOCIATION_TYPES_KEY = "uddi:d612efe1-f11f-3bf2-f899-e0e21e952a87";

	/**
	 * The name of the classification scheme of association types.
	 * Association type concepts are direct children of this classification
	 * scheme.
	 */
	public static final String ASSOCIATION_TYPES_NAME = "AssociationType";

	/**
	 * The qualified name of the "attributeKey" slot:
	 * "{http://namespaces.CentraSite.com/Schema}attributeKey"
	 */
	public static final String SLOT_NAME_ATTRIBUTES_KEY = "{http://namespaces.CentraSite.com/Schema}attributeKey";
	
	private static final CentraSiteRegistryInfo theInstance = new CentraSiteRegistryInfo();

	/**
	 * Returns the singleton instance.
	 */
	public static CentraSiteRegistryInfo getInstance() {
		return theInstance;
	}

	/**
	 * Private constructor, to ensure singleton pattern.
	 */
	private CentraSiteRegistryInfo() {
		// Does nothing.
	}

	@Override
	public String getAssociationTypesKey() {
		return ASSOCIATION_TYPES_KEY;
	}

	@Override
	public String getAssociationTypesName() {
		return ASSOCIATION_TYPES_NAME;
	}

	@Override
	public String getObjectTypesKey() {
		return OBJECT_TYPES_KEY;
	}

	@Override
	public String getObjectTypesName() {
		return OBJECT_TYPES_NAME;
	}

    @Override
    public MetaModelAccessor getMetaModelAccessor() {
        return CentraSiteMetaModelAccessor.getInstance();
    }

    @Override
    public InnerModelAccessor getModelAccessor() {
        return CentraSiteInnerModelAccessor.getInstance();
    }
}
