package net.sf.csutils.core.registry.jaxmas;

import java.util.Properties;

import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.JAXRException;

import net.sf.csutils.core.registry.ConnectionProvider;

import org.apache.labs.jaxmas.registry.ConnectionFactoryImpl;


public class JaxMasConnectionProvider implements ConnectionProvider {
    private String driver, url, user, password, locale;

    /**
     * Returns the database driver to use.
     * @return Name of the database driver.
     */
    public String getDriver() {
        return driver;
    }

    /**
     * Sets the database driver to use.
     * @param pDriver Name of the database driver to use.
     */
    public void setDriver(String pDriver) {
        driver = pDriver;
    }

    /**
     * Returns the database URL to use.
     * @return The database URL to use.
     */
    public String getUrl() {
        return url;
    }

    /**
     * Sets the database URL to use.
     * @param pURL The database URL to use.
     */
    public void setUrl(String pURL) {
        url = pURL;
    }

    /**
     * Returns the database user to use.
     * @return The database user to use.
     */
    public String getUser() {
        return user;
    }

    /**
     * Sets the database user to use.
     * @param pUser The database user to use.
     */
    public void setUser(String pUser) {
        user = pUser;
    }

    /**
     * Returns the database users password.
     * @return The database users password.
     */
    public String getPassword() {
        return password;
    }

    /**
     * Sets the database users password.
     * @param pPassword The database users password to use.
     */
    public void setPassword(String pPassword) {
        password = pPassword;
    }

    /**
     * Returns the locale.
     * @return Returns the locale to use.
     */
    public String getLocale() {
        return locale;
    }

    /**
     * Sets the locale.
     * @param pLocale Sets the locale to use.
     */
    public void setLocale(String pLocale) {
        locale = pLocale;
    }

    private void setProperty(Properties pProps, String pKey, String pValue) {
        if (pValue != null) {
            pProps.setProperty(pKey, pValue);
        }
    }

    @Override
    public Connection getConnection() throws JAXRException {
        System.setProperty("javax.xml.registry.ConnectionFactoryClass",
                ConnectionFactoryImpl.class.getName());
        final ConnectionFactory connFactory = ConnectionFactory.newInstance();
        final Properties props = new Properties();
        setProperty(props, ConnectionFactoryImpl.PROPERTY_DRIVER, driver);
        setProperty(props, ConnectionFactoryImpl.PROPERTY_URL, url);
        setProperty(props, ConnectionFactoryImpl.PROPERTY_USER, user);
        setProperty(props, ConnectionFactoryImpl.PROPERTY_PASSWORD, password);
        setProperty(props, ConnectionFactoryImpl.PROPERTY_LOCALE, locale);
        connFactory.setProperties(props);
        return connFactory.createConnection();
    }

}
