package net.sf.csutils.core.registry.jaxmas;

import java.util.HashSet;
import java.util.Set;

import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.JAXRException;
import javax.xml.registry.LifeCycleManager;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.RegistryObject;

import net.sf.csutils.core.registry.RegistryFacade;
import net.sf.csutils.core.registry.impl.AbstractInnerModelAccessor;
import net.sf.csutils.core.registry.impl.InnerModelAccessor;

import org.apache.labs.jaxmas.registry.accessor.ROAccessors.ObjectTypes;


/**
 * Implementation of {@link InnerModelAccessor} for JaxMas.
 */
public class JaxMasInnerModelAccessor extends AbstractInnerModelAccessor {
    private static final JaxMasInnerModelAccessor theInstance = new JaxMasInnerModelAccessor();
    private static final Set<String> BUILTIN_TYPES = new HashSet<String>();
    static {
        for (ObjectTypes ot : ObjectTypes.values()) {
            BUILTIN_TYPES.add(ot.getInterfaceName());
        }
    }

    private JaxMasInnerModelAccessor() {
        // Private constructor, to ensure singleton pattern.
    }

    /**
     * Returns the singleton instance
     * @return The singleton instance
     */
    public static JaxMasInnerModelAccessor getInstance() {
        return theInstance;
    }

    @SuppressWarnings("unchecked")
    @Override
    public <RO extends RegistryObject> RO createInstance(
            RegistryFacade pFacade, String pType) throws JAXRException {
        final BusinessLifeCycleManager blcm = pFacade.getBusinessLifeCycleManager();
        if (BUILTIN_TYPES.contains(pType)) {
            return (RO) blcm.createObject(pType);
        }
        final RegistryObject ro = (RegistryObject) blcm.createObject(LifeCycleManager.REGISTRY_ENTRY);
        final Concept objectType = pFacade.findObjectType(pType);
        if (objectType == null) {
            throw new JAXRException("No such object type: " + pType);
        }
        ro.addClassification(blcm.createClassification(objectType));
        return (RO) ro;
    }
}
