package net.sf.csutils.core.tests;

import java.io.IOException;
import java.io.InputStream;
import java.net.PasswordAuthentication;
import java.net.URL;
import java.util.HashSet;
import java.util.Properties;
import java.util.Set;

import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.JAXRException;

import net.sf.csutils.core.registry.RegistryFacade;
import net.sf.csutils.core.registry.SimpleRegistryFacade;

import org.apache.log4j.PropertyConfigurator;

import com.centrasite.jaxr.ConnectionFactoryImpl;


/**
 * Abstract base class for implementing tests. This class is intentionally
 * not dependent on JUnit, so that JUnit doesn't appear in the dependency
 * list. OTOH, this means that any test class must contain something
 * similar to
 * <pre>
 *   @BeforeClass public static void setUp() {
 *       initProperties();
 *   }
 * </pre>
 */
public abstract class AbstractTestCase {
	protected static URL getTopLevelResource(String pResource) {
		final URL url = Thread.currentThread().getContextClassLoader().getResource(pResource);
		if (url == null) {
			throw new IllegalStateException("Unable to locate resource: " + pResource);
		}
		return url;
	}

	private static String centraSiteUrl, centraSiteUser, centraSitePassword;

	public static void init() throws IOException {
		initProperties();
	}
	
	public static void initProperties() throws IOException {
		InputStream istream = getTopLevelResource("cs-utils-core.properties.xml").openStream();
		try {
			final Properties props = new Properties();
			props.loadFromXML(istream);
			centraSiteUrl = props.getProperty("centrasite.url");
			centraSiteUser = props.getProperty("centrasite.user");
			centraSitePassword = props.getProperty("centrasite.password");
			istream.close();
			istream = null;
		} finally {
			if (istream != null) { try { istream.close(); } catch (Throwable t) { /* Ignore me */ } }
		}
		final URL url = getTopLevelResource("log4j.properties");
		PropertyConfigurator.configure(url);
	}

	protected static RegistryFacade openConnection() throws JAXRException {
		System.setProperty("javax.xml.registry.ConnectionFactoryClass", ConnectionFactoryImpl.class.getName());
		ConnectionFactory connFactory = ConnectionFactory.newInstance();
		Properties p = new Properties();
		p.setProperty("javax.xml.registry.queryManagerURL", centraSiteUrl);
		connFactory.setProperties(p);
		Connection connection = connFactory.createConnection();
		Set<Object> credentials = new HashSet<Object>(1);
		credentials.add(new PasswordAuthentication(centraSiteUser, centraSitePassword.toCharArray()));
		connection.setCredentials(credentials);
		return new SimpleRegistryFacade(connection);
	}
}
